%
% Simulation program to realize a simple DS-CDMA transmission 
% Random Data -> Spreading -> Tx Filtering -> Canal
% -> Rx Filtering -> Despreading -> Correlation -> Estimated Data 
% -> BER and quantization loss computation 
%
% Programmed by O. Sentieys
% [H. Harada and R. Prasda, Simulation and Software Radio for Mobile Communications, Artech House, 2002]

%******************** Preparation part *************************************
close all;
clear all;
Fs=256000.0;        % Symbol rate
sr=Fs;
ml   = 2;           % number of modulation levels
br=sr * ml;         % Bit rate
Nd = 200;           % Number of symbols that simulates in each loop
ebn0=10000;         % Eb/N0
R=2;                % Number of oversamples

%************************* Filter initialization ***************************

N=16;                       % Number of taps
alpha=0.22;                 % Rolloff factor
[Tx_fir] = hrollfcoef(N,R,sr,alpha,1);   %Transmitter filter coefficients
dyn_Tx=sum(abs(Tx_fir));
[Rx_fir] = hrollfcoef(N,R,sr,alpha,0);   %Receiver filter coefficients 
dyn_Rx=sum(abs(Rx_fir));

%************************* Filter coefficient quantization ****************
b_h_tx=8;                 % number of bit for transmitter filter coefficients
b_tx=10;                  % number of bit for transmitter filter output 
b_CAN=12;                 % number of bit for receiver filter input  
b_h_rx=12;                % number of bit for receiver filter coefficients
b_rx=b_h_rx+b_CAN;    % number of bit for receiver filter output  

Tx_fir_q = round(Tx_fir.*2^(b_h_tx-1));
Rx_fir_q = round(Rx_fir.*2^(b_h_rx-1));

Tx_fir = Tx_fir_q.*2^(1-b_h_tx);
Rx_fir = Rx_fir_q.*2^(1-b_h_rx);

%********************** Spreading code initialization **********************

user  = 1;                                                          % number of users
seq   = 3;                                                          % 1:M-sequence  2:Gold  3:Orthogonal Gold
stage = 3;                                                          % number of stages
ptap1 = [1 3];                                                      % position of taps for 1st
ptap2 = [2 3];                                                      % position of taps for 2nd
regi1 = [1 1 1];                                                    % initial value of register for 1st
regi2 = [1 1 1];                                                    % initial value of register for 2nd

%******************** Generation of the spreading code *********************

switch seq
case 1                                                              % M-sequence
    code = mseq(stage,ptap1,regi1,user);
case 2                                                              % Gold sequence
    m1   = mseq(stage,ptap1,regi1);
    m2   = mseq(stage,ptap2,regi2);
    code = goldseq(m1,m2,user);
case 3                                                              % Orthogonal Gold sequence
    m1   = mseq(stage,ptap1,regi1);
    m2   = mseq(stage,ptap2,regi2);
    code = [goldseq(m1,m2,user),zeros(user,1)];
end
code = code * 2 - 1;
SF = length(code);                                                  % Sprading factor
clen= SF;                                                      

%**************************** START CALCULATION ****************************

nloop = 1;                                                       % simulation number of times
noe   = 0;
nod   = 0;

%for delay=0:R*SF-1
for ii=1:nloop
    
%****************************** Transmitter ********************************
    data = rand(user,Nd*ml) > 0.5;
    %save data data
    %load data
    %data = zeros(user,Nd*ml);
    
    [ich, qch]  = qpskmod(data,user,Nd,ml);                         % QPSK modulation
    [ich1,qch1] = spread(ich,qch,code);                             % spreading
    
    [h,v] = size(ich1);
    ich2 = zeros(h,v*R);
    qch2 = zeros(h,v*R);
    ich2(:,1:R:1+R*(v-1)) = ich1;                                   % over sampling real part
    qch2(:,1:R:1+R*(v-1)) = qch1;                                   % over sampling imag part


    ich3 = conv2(ich2,Tx_fir);                                      % filter real part
    qch3 = conv2(qch2,Tx_fir);                                      % filter imag part
    
    if user == 1                                                    % transmission
        ich4 = ich3;
        qch4 = qch3;
    else
        ich4 = sum(ich3);
        qch4 = sum(qch3);
    end
        
%******************************** Receiver *********************************
  
    spow = sum(rot90(ich3.^2 + qch3.^2)) / Nd;                      % attenuation Calculation
    attn = sqrt(0.5 * spow * sr / br * 10^(-ebn0/10));
    %[ich5,qch5] = addWGN(ich4,qch4,attn);                           % Add White Gaussian Noise (AWGN)
    
    %b=[381 1193 384 -2527 -3866 1438 13051 23067 23067 13051 1438 -3866 -2527 384 1193 381];
    %b=2^(-16).*b;
    b = [1 0 0 0 0 0 0.5 0 0 0 0.3 0 0 0 0 0];                      % Channel filtering (3 paths)
    %[h,w] = freqz(b,1,256); 
    %figure; plot(w,20*log10(abs(h)),'b'); title('Log Magnitude RIF channel filter'); 
    %[ich5,qch5] = addWGN(filter(b, 1, ich4),filter(b, 1, qch4),attn);       % Add White Gaussian Noise (AWGN)
 
    delay = 0; % Testing the influence of the delay

    ich5(:,:) = ich3(:,delay+1:length(ich3));     
    qch5(:,:) = qch3(:,delay+1:length(qch3)); 
    
    
    
    ich6 = conv2(ich5,Rx_fir);                                      % filter real part
    qch6 = conv2(qch5,Rx_fir);                                      % filter imag part

    
    sampl = N * R + 1;                                              % optimal sample time          
    ich7  = ich6(:,sampl:R:R*Nd*SF+sampl-1);                        % down sample real part
    qch7  = qch6(:,sampl:R:R*Nd*SF+sampl-1);                        % down sample imag part
    
    [ich8 qch8] = despread(ich7,qch7,code);                         % despreading
    demodata = qpskdemod(ich8,qch8,user,Nd,ml);                     % QPSK demodulation
    
%************************** Bit Error Rate (BER) ***************************

    noe2 = sum(sum(abs(data-demodata)));
    nod2 = user * Nd * ml;
    noe  = noe + noe2;
    nod  = nod + nod2;
    
    fprintf('%d\t%e\n',ii,noe2/nod2);

    %******************************** PLOT ********************************
    NPLOT = Nd-1;
    figure;
    subplot(2,1,1); stairs(data(1,1:2*NPLOT+1)); title('Input Data (user 1)'); AXIS([1 2*NPLOT+1 -0.1 1.1]);
    subplot(2,1,2); stairs(demodata(1,1:2*NPLOT+1)); title('Demodulated Data'); AXIS([1 2*NPLOT+1 -0.1 1.1]);
    
    %clear ich5 qch5;
end
%end


fig=0;
if (fig == 1) 
figure; 
SUBPLOT(3,1,1); stairs(data(1,1:2*NPLOT+1)); title('Input Data (user 1)'); AXIS([1 2*NPLOT+1 -0.1 1.1]);
subplot(3,1,2); stairs(ich(1,1:NPLOT+1)); title('Transmitted Data after modulation (Real Part)'); AXIS([1 NPLOT+1 -1.1 1.1]);
subplot(3,1,3); stairs(qch(1,1:NPLOT+1)); title('Transmitted Data after modulation (Imag Part)'); AXIS([1 NPLOT+1 -1.1 1.1]);
figure; 
SUBPLOT(3,1,1); stem(data(1,1:2*NPLOT+1)); title('Input Data (user 1)'); AXIS([1 2*NPLOT+1 0 1]);
subplot(3,1,2); stem(ich(1,1:NPLOT+1)); title('Transmitted Data after modulation (Real Part)');
subplot(3,1,3); stem(qch(1,1:NPLOT+1)); title('Transmitted Data after modulation (Imag Part)');

figure; 
SUBPLOT(5,1,1); stairs(code(1,1:SF)); title('Spreading Code (user 1)'); AXIS([1 SF+1 -1.1 1.1]);
subplot(5,1,2); stairs(ich1(1,1:NPLOT*SF)); title('Transmitted Data after spreading and modulation (Real Part)'); AXIS([1 NPLOT*SF -1.1 1.1]);
subplot(5,1,3); stairs(qch1(1,1:NPLOT*SF)); title('Transmitted Data after spreading and modulation (Imag Part)'); AXIS([1 NPLOT*SF -1.1 1.1]);
subplot(5,1,4); stairs(ich1(1,1:SF*2+1)); title('Transmitted Data after spreading and modulation (Real Part), 2 first symbols'); AXIS([1 2*SF+1 -1.1 1.1]);
subplot(5,1,5); stairs(qch1(1,1:SF*2+1)); title('Transmitted Data after spreading and modulation (Imag Part), 2 first symbols'); AXIS([1 2*SF+1 -1.1 1.1]);
figure; 
SUBPLOT(5,1,1); stem(code(1,1:SF)); title('Spreading Code (user 1)'); AXIS([1 SF -1.1 1.1]);
subplot(5,1,2); stem(ich1(1,1:NPLOT*SF)); title('Transmitted Data after spreading and modulation (Real Part)'); AXIS([1 NPLOT*SF -1.1 1.1]);
subplot(5,1,3); stem(qch1(1,1:NPLOT*SF)); title('Transmitted Data after spreading and modulation (Imag Part)'); AXIS([1 NPLOT*SF -1.1 1.1]);
subplot(5,1,4); stem(ich1(1,1:SF*2+1)); title('Transmitted Data after spreading and modulation (Real Part), 2 first symbols'); AXIS([1 2*SF+1 -1.1 1.1]);
subplot(5,1,5); stem(qch1(1,1:SF*2+1)); title('Transmitted Data after spreading and modulation (Imag Part), 2 first symbols'); AXIS([1 2*SF+1 -1.1 1.1]);

figure; 
subplot(4,1,1); stem(ich2(1,1:NPLOT*SF*R)); title('Transmitted Data after modulation and oversampling (Real Part)');
subplot(4,1,2); stem(qch2(1,1:NPLOT*SF*R)); title('Transmitted Data after modulation and oversampling (Imag Part)');
subplot(4,1,3); stem(ich2(1,1:2*SF*R+1)); title('Transmitted Data after modulation and oversampling (Real Part), 2 first symbols');
subplot(4,1,4); stem(qch2(1,1:2*SF*R+1)); title('Transmitted Data after modulation and oversampling (Imag Part), 2 first symbols');

figure; 
subplot(4,1,1); stem(ich3(1,1:NPLOT^2*SF*R)); title('Transmitted Data after filtering (Real Part)');
subplot(4,1,2); stem(qch3(1,1:NPLOT^2*SF*R)); title('Transmitted Data after filtering (Imag Part)');
subplot(4,1,3); stem(ich3(1,1:4*SF*R+1)); title('Transmitted Data after filtering (Real Part), 4 first symbols');
subplot(4,1,4); stem(qch3(1,1:4*SF*R+1)); title('Transmitted Data after filtering (Imag Part), 4 first symbols');

figure; 
subplot(4,1,1); stem(Tx_fir); title('Tx FIR coefficients');
subplot(4,1,2); stem(Rx_fir); title('Rx FIR coefficients');
subplot(4,1,3); stem(Tx_fir_q); title('Tx FIR coefficients (quantized)');
subplot(4,1,4); stem(Rx_fir_q); title('Rx FIR coefficients (quantized)');

%figure; 
%subplot(2,1,1); stem(ich4(1,1:NPLOT^2*SF*R)); title('Transmitted Data (all users) (Real Part)');
%subplot(2,1,2); stem(qch4(1,1:NPLOT^2*SF*R)); title('Transmitted Data (all users) (Imag Part)');

figure; 
subplot(4,1,1); stem(ich5(1,1:NPLOT^2*SF*R)); title('Received Data (Real Part)');
subplot(4,1,2); stem(qch5(1,1:NPLOT^2*SF*R)); title('Received Data (Imag Part)');
subplot(4,1,3); stem(ich5(1,1:4*SF*R+1)); title('Received Data (Real Part), 4 first symbols');
subplot(4,1,4); stem(qch5(1,1:4*SF*R+1)); title('Received Data (Imag Part), 4 first symbols');

%figure; 
%subplot(2,1,1); stem(ich6(1,1:NPLOT^2*SF*R)); title('Received Data after filtering (Real Part)');
%subplot(2,1,2); stem(qch6(1,1:NPLOT^2*SF*R)); title('Received Data after filtering (Imag Part)');

figure; 
subplot(4,1,1); stem(ich7(1,1:NPLOT^2*SF)); title('Received Data after filtering and downsampling (Real Part)');
subplot(4,1,2); stem(qch7(1,1:NPLOT^2*SF)); title('Received Data after filtering and downsampling (Imag Part)');
subplot(4,1,3); stem(ich7(1,1:4*SF+1)); title('Received Data after filtering and downsampling (Real Part), 4 first symbols');
subplot(4,1,4); stem(qch7(1,1:4*SF+1)); title('Received Data after filtering and downsampling (Imag Part), 4 first symbols');

figure; 
subplot(3,1,1); stem(ich8(1,1:NPLOT+1)); title('Received Data after despreading (Real Part)'); AXIS([1 NPLOT+1 -10 10]);
subplot(3,1,2); stem(qch8(1,1:NPLOT+1)); title('Received Data after despreading (Imag Part)'); AXIS([1 NPLOT+1 -10 10]);
subplot(3,1,3); stairs(demodata(1,1:2*NPLOT+1)); title('Demodulated Data'); AXIS([1 2*NPLOT+1 -0.1 1.1]);
end


ber = noe / nod;
fprintf('Eb/N0: %d;\tNb of err: %d;\tNb of data: %d;\tBER: %e\n',ebn0,noe,nod,noe/nod);                   % fprintf: built in function
fid = fopen('BER.dat','w');
fprintf(fid,'%d\t%e\t%f\t%f\t\n',ebn0,noe/nod,noe,nod);             % fprintf: built in function
fclose(fid);

%*************************** DATA FILES ***********************************

fid = fopen('DATA.dat','w');
fprintf(fid,'Input Data\n%d\n',Nd*ml);
for ii=1:Nd*ml
    fprintf(fid,'%d\n',data(1,ii));
end
fclose(fid);

fid = fopen('MOD_DATA.dat','w');
fprintf(fid,'Modulated Data\n%d\n',Nd);
for ii=1:Nd
    fprintf(fid,'%d  %d\n',ich(1,ii),qch(1,ii));
end
fclose(fid);

fid = fopen('DATA1.dat','w');
fprintf(fid,'Modulated and spreaded Data: ich1   qch1\n%d\n',Nd*SF);
for ii=1:Nd*SF
    fprintf(fid,'%d  %d\n',ich1(1,ii),qch1(1,ii));
end
fclose(fid);

fid = fopen('DATA2.dat','w');
fprintf(fid,'Oversampled Data: ich2   qch2\n%d\n',length(ich2));
for ii=1:length(ich2)
    fprintf(fid,'%d  %d\n',ich2(1,ii),qch2(1,ii));
end
fclose(fid);

fid = fopen('DATA3.dat','w');
fprintf(fid,'Filtered Data: ich3   qch3\n%d\n',length(ich3));
for ii=1:length(ich3)
    fprintf(fid,'%d  %d\n',round(ich3(1,ii)*2^(b_tx-1-2)),round(qch3(1,ii)*2^(b_tx-1-2)));
    % 2 bits for integer part
end
fclose(fid);

dyn_rx_signal = ceil(log2(max(ich5(:))));
fid = fopen('DATA5.dat','w');
fprintf(fid,'Received Data: ich5   qch5\n%d\n',length(ich5));
for ii=1:length(ich5)
    fprintf(fid,'%d  %d\n',round(ich5(1,ii)*2^(b_CAN-1)),round(qch5(1,ii)*2^(b_CAN-1)));
end
fclose(fid);

fid = fopen('DATA7.dat','w');
fprintf(fid,'Filtered Data after downsampling: ich7   qch7\n%d\n',length(ich7));
for ii=1:length(ich7)
    fprintf(fid,'%d  %d\n',round(ich7(1,ii)*2^(b_rx-2)),round(qch7(1,ii)*2^(b_rx-2)));
end
fclose(fid);

fid = fopen('DEMODATA.dat','w');
fprintf(fid,'Output Data\n%d\n',Nd*ml);
for ii=1:Nd*ml
    fprintf(fid,'%d\n',demodata(1,ii));
end
fclose(fid);

 % DATA for test of Tx filter including the intialisation of the vector
 % to -1
 test_ich1 = zeros(1,length(ich1(1,:)));
 test_ich1(1:N) = -1;
 test_ich1(N+1:length(ich1(1,:))) = ich1(1,1:length(ich1(1,:))-N);
 test_qch1 = zeros(1,length(qch1(1,:)));
 test_qch1(1:N) = -1;
 test_qch1(N+1:length(qch1(1,:))) = qch1(1,1:length(qch1(1,:))-N);
 test_ich2 = zeros(1,v*R);
 test_qch2 = zeros(1,v*R);
 test_ich2(1:R:1+R*(v-1)) = test_ich1; 
 test_qch2(1:R:1+R*(v-1)) = test_qch1; 
 test_ich3 = conv2(test_ich2,Tx_fir_q); 
 test_qch3 = conv2(test_qch2,Tx_fir_q); 
 fid = fopen('TEST_DATA3.dat','w');
 fprintf(fid,'Filtered Data (for test): ich3   qch3\n%d\n',length(test_ich3));
 for ii=R*N:length(test_ich3)
    fprintf(fid,'%d  %d\n',test_ich3(1,ii),test_qch3(1,ii));
    % 2 bits for integer part
 end
 fclose(fid);
 
 % DATA for test of Rx filter : impulse response followed by step
 % response
 test_ich5 = zeros(1,length(ich5(1,:))); 
 test_ich5(1) = 0.5;
 test_ich5(40:length(ich5(1,:))) = 0.5;
 test_qch5 = zeros(1,length(qch5(1,:)));
 test_qch5(1) = 0.25; 
 test_qch5(40:length(qch5(1,:))) = 0.25;
 test_ich6 = conv2(test_ich5,Rx_fir); 
 test_qch6 = conv2(test_qch5,Rx_fir); 
 test_ich7 = test_ich6(1:2:length(test_ich6)); 
 test_qch7 = test_qch6(1:2:length(test_qch6));
 fid = fopen('TEST_DATA5.dat','w');
 fprintf(fid,'Received Data (for test): ich5   qch5\n%d\n',length(test_ich5));
 for ii=1:length(test_ich5)
    fprintf(fid,'%d  %d\n',round(test_ich5(1,ii)*2^(b_CAN-1)),round(test_qch5(1,ii)*2^(b_CAN-1)));
 end
 fclose(fid);
 fid = fopen('TEST_DATA7.dat','w');
 fprintf(fid,'Filtered Data (for test): ich7   qch7\n%d\n',length(test_ich7));
 for ii=1:length(test_ich7)
    fprintf(fid,'%d  %d\n',round(test_ich7(1,ii)*2^(b_rx-2)),round(test_qch7(1,ii)*2^(b_rx-2)));
 end
 fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fid = fopen('CDMA_pack.vhd','w');
fprintf(fid,'package CDMA_pack is\n');

% Input data
fprintf(fid,'constant input_data : array (0 to %d) of integer := (',Nd*ml);
for ii=1:Nd*ml-1
    fprintf(fid,'%d, ',data(1,ii));
end
fprintf(fid,'%d);\n\n',data(1,ii+1));

% Spreading code
fprintf(fid,'constant SF : integer := %d;\n',SF);
fprintf(fid,'constant code : array (0 to %d) of integer := (',SF-1);
for ii=1:SF-1
    fprintf(fid,'%d, ',(code(1,ii)+1)/2);
end
fprintf(fid,'%d);\n\n',(code(1,ii)+1)/2);

% Tx Fir coefficients
fprintf(fid,'constant H_Tx_fir : array (0 to %d) of integer := (',length(Tx_fir_q)-1);
for ii=1:length(Tx_fir_q)-1
    fprintf(fid,'%d, ',Tx_fir_q(ii));
end
fprintf(fid,'%d);\n\n',Tx_fir_q(ii+1));

% Rx Fir coefficients
fprintf(fid,'constant H_Rx_fir : array (0 to %d) of integer := (',length(Tx_fir_q)-1);
for ii=1:length(Rx_fir_q)-1
    fprintf(fid,'%d, ',Rx_fir_q(ii));
end
fprintf(fid,'%d);\n\n',Rx_fir_q(ii+1));



fprintf(fid,'end CDMA_pack;\n');
fclose(fid);

%******************************** end of file ********************************
